# Generated by Django 3.1.4 on 2020-12-06 18:59

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='LogEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=200)),
                ('description', models.TextField(blank=True, null=True)),
                ('action', models.CharField(choices=[('release', 'RELEASE'), ('remove', 'REMOVE'), ('anticipate', 'ANTICIPATE'), ('improved', 'IMPROVED')], default='r', max_length=20)),
                ('version', models.CharField(blank=True, max_length=20, null=True)),
            ],
            options={
                'verbose_name': 'Log Entry',
                'verbose_name_plural': 'Log Entries',
                'ordering': ('-version',),
            },
        ),
        migrations.CreateModel(
            name='Changelog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('version', models.CharField(max_length=20)),
                ('title', models.CharField(default='', max_length=200)),
                ('description', models.TextField(blank=True, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('published', models.BooleanField(default=True)),
                ('items', models.ManyToManyField(related_name='changelog', to='changelogs.LogEntry')),
            ],
            options={
                'verbose_name': 'Changelog',
                'verbose_name_plural': 'Changelogs',
                'ordering': ('-created_at',),
            },
        ),
    ]



from django.db import models


LOG_CHOICES = (
    ('release', 'RELEASE'),
    ('remove', 'REMOVE'),
    ('anticipate', 'ANTICIPATE'),
    ('improved', 'IMPROVED'),
)
LOG_DICT = dict(LOG_CHOICES)


class LogEntry(models.Model):
    class Meta:
        verbose_name = 'Log Entry'
        verbose_name_plural = 'Log Entries'
        ordering = ('-version',)
    title = models.CharField(max_length=200)
    description = models.TextField(null=True, blank=True)
    action = models.CharField(max_length=20, choices=LOG_CHOICES, default='r')
    version = models.CharField(max_length=20, null=True, blank=True)


    def __str__(self):
        return f'[{LOG_DICT.get(self.action, self.action.upper())}] {self.title}'
    
    def save(self, *args, **kwargs):
        try:
            cl = self.changelog.first()
            if cl: self.version = cl.version
        except:
            pass
        super().save(*args, **kwargs)

class Changelog(models.Model):
    class Meta:
        verbose_name = 'Changelog'
        verbose_name_plural = 'Changelogs'
        ordering = ('-created_at',)
    version = models.CharField(max_length=20)
    title = models.CharField(default='', max_length=200)
    description = models.TextField(null=True, blank=True)
    items = models.ManyToManyField(LogEntry, related_name='changelog')
    created_at = models.DateTimeField(auto_now_add=True)
    published = models.BooleanField(default=True)

    @classmethod
    def latest(cls):
        return cls.objects.order_by('-created_at').prefetch_related('items').first()
    
    def __str__(self):
        return f'{self.version}: {self.title}'









from django.contrib import admin
from .models import *


class LogEntryAdmin(admin.ModelAdmin):
    model = LogEntry
    list_display = ['title', 'action', 'version']
    readonly_fields = ['version']

    def get_queryset(self, request):
        return LogEntry.objects.all()

class ChangelogAdmin(admin.ModelAdmin):
    model = Changelog
    actions = ['publish']
    list_display = ['title', 'version', 'published', 'created_at', 'items_count']

    def get_queryset(self, request):
        return Changelog.objects.all().prefetch_related('items')
    
    def items_count(self, instance):
        return instance.items.count()

    def publish(self, request, qs):
        qs.update(published=True)
    publish.short_description = 'Publish selected Changelogs'


admin.site.register(Changelog, ChangelogAdmin)
admin.site.register(LogEntry, LogEntryAdmin)

